@@ -62,11 +62,11 @@
 int kent_pool_size;
 
 /* Background/daemon mode */
-bool am_daemon;				/* True if we went daemon */
+bool am_daemon = false;			/* True if we went daemon */
 bool msg_squash = false;		/* True if we want no messages on the console */
 bool quiet = false;			/* True if we want no console output at all */
 volatile bool server_running = true;	/* set to false, to stop daemon */
-
+bool do_reseed = false;			/* force a reseed event */
 bool ignorefail = false; /* true if we ignore MAX_RNG_FAILURES */
 
 /* Command line arguments and processing */
@@ -115,7 +115,7 @@ static struct argp_option options[] = {
 	  "Number of bytes written to random-device at a time (default: 64)" },
 
 	{ "fill-watermark", 'W', "n", 0,
-	  "Do not stop feeding entropy to random-device until at least n bits of entropy are available in the pool (default: 2048), 0 <= n <= 4096" },
+	  "Do not stop feeding entropy to random-device until at least n bits of entropy are available in the pool (default: 3/4 of poolsize), 0 <= n <= 4096" },
 
 	{ "quiet", 'q', 0, 0, "Suppress all messages" },
 
@@ -123,6 +123,8 @@ static struct argp_option options[] = {
 
 	{ "entropy-count", 'e', "n", 0, "Number of entropy bits to support (default: 8), 1 <= n <= 8" },
 
+	{ "force-reseed", 'R', "n", 0, "Time in seconds to force adding entropy to the random device" },
+
 	{ 0 },
 };
 
@@ -130,11 +132,13 @@ static struct arguments default_arguments = {
 	.random_name	= "/dev/random",
 	.pid_file	= "/var/run/rngd.pid",
 	.random_step	= 64,
+	.fill_watermark = -1,
 	.daemon		= true,
 	.test		= false,
 	.list		= false,
 	.ignorefail	= false,
 	.entropy_count	= 8,
+	.force_reseed	= 60 * 5,
 };
 struct arguments *arguments = &default_arguments;
 
@@ -149,6 +153,7 @@ static enum {
 	ENT_TPM = 1,
 	ENT_RDRAND,
 	ENT_DARN,
+	ENT_RNDR,
 	ENT_NISTBEACON,
 	ENT_JITTER,
 	ENT_PKCS11,
@@ -210,6 +215,11 @@ static struct rng_option jitter_options[] = {
 		.type = VAL_INT,
 		.int_val = 1,
 	},
+	[JITTER_OPT_FORCE_INT_TIMER] = {
+		.key = "force_soft_timer",
+		.type = VAL_INT,
+		.int_val = 0,
+	},
 	{
 		.key = NULL,
 	}
@@ -235,6 +245,17 @@ static struct rng_option pkcs11_options[] = {
 	}
 };
 
+static struct rng_option nist_options[] = {
+	[NIST_OPT_USE_AES] = {
+		.key = "use_aes",
+		.type = VAL_INT,
+		.int_val = 1,
+	},
+	{
+		.key = NULL,
+	}
+};
+
 static struct rng_option rtlsdr_options[] = {
 	[RTLSDR_OPT_DEVID] = {
 		.key = "device_id",
@@ -260,6 +281,9 @@ static struct rng_option rtlsdr_options[] = {
 		.key = "sample_max",
 		.type = VAL_INT,
 		.int_val = 2800000,
+	},
+	{
+		.key = NULL,
 	}
 };
 
@@ -313,6 +337,19 @@ static struct rng entropy_sources[ENT_MAX] = {
 #endif
 		.rng_options	= darn_options,
 	},
+	{
+		.rng_name       = "ARM v8.5 RNDR Instruction RNG",
+		.rng_sname	= "rndr",
+		.rng_fd	 = -1,
+		.flags		= { 0 },
+#ifdef HAVE_RNDR
+		.xread	  = xread_rndr,
+		.init	   = init_rndr_entropy_source,
+#else
+		.disabled	= true,
+#endif
+		.rng_options	= drng_options,
+	},
 	{
 		.rng_name	= "NIST Network Entropy Beacon",
 		.rng_sname	= "nist",
@@ -325,7 +362,7 @@ static struct rng entropy_sources[ENT_MAX] = {
 		.init		= init_nist_entropy_source,
 #endif
 		.disabled	= true,
-		.rng_options	= NULL,
+		.rng_options	= nist_options,
 	},
 	{
 		.rng_name	= "JITTER Entropy generator",
@@ -402,7 +439,7 @@ static int find_ent_src_idx(const char *name_idx)
 	} else {
 		idx = strtoul(name_idx, NULL, 10);
 		if ((idx == LONG_MAX) || (idx >= ENT_MAX)) {
-			message(LOG_CONS|LOG_INFO, "option index out of range: %u\n", idx);
+			message(LOG_CONS|LOG_INFO, "option index out of range: %d\n", idx);
 			return -ERANGE;
 		}
 		message(LOG_CONS|LOG_INFO, "Note, reference of entropy sources by index "
@@ -493,7 +530,7 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 			}
 			options++;
 		}
-		message(LOG_CONS|LOG_INFO, "Option %s not found for source idx %lu\n", optkey, idx);
+		message(LOG_CONS|LOG_INFO, "Option %s not found for source idx %ld\n", optkey, idx);
 		return -ERANGE;
 		break;
 
@@ -503,7 +540,7 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 			return idx;
 
 		entropy_sources[idx].disabled = true;
-		message(LOG_CONS|LOG_INFO, "Disabling %lu: %s (%s)\n", idx,
+		message(LOG_CONS|LOG_INFO, "Disabling %ld: %s (%s)\n", idx,
 			entropy_sources[idx].rng_name, entropy_sources[idx].rng_sname);
 		break;
 	case 'n':
@@ -512,7 +549,7 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 			return idx;
 
 		entropy_sources[idx].disabled = false;
-		message(LOG_CONS|LOG_INFO, "Enabling %lu: %s (%s)\n", idx,
+		message(LOG_CONS|LOG_INFO, "Enabling %ld: %s (%s)\n", idx,
 			entropy_sources[idx].rng_name, entropy_sources[idx].rng_sname);
 		break;
 	case 'l':
@@ -563,6 +600,14 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 			arguments->entropy_count = e;
 		break;
 	}
+	case 'R': {
+		int R;
+		if ((sscanf(arg,"%i", &R) == 0) || (R < 0))
+			argp_usage(state);
+		else
+			arguments->force_reseed = R;
+		break;
+	}
 	default:
 		return ARGP_ERR_UNKNOWN;
 	}
@@ -588,6 +633,10 @@ static int update_kernel_random(struct rng *rng, int random_step,
 		 p += random_step) {
 		if (!server_running)
 			return 0;
+		if (do_reseed) {
+			do_reseed = false;
+			alarm(arguments->force_reseed);
+		}
 		rc = random_add_entropy(p, random_step);
 		if (rc == -1)
 			return 1;
@@ -731,6 +780,11 @@ static void term_signal(int signo)
 	server_running = false;
 }
 
+static void force_reseed_signal(int signo)
+{
+	do_reseed = true;
+}
+
 static void alarm_signal(int signo)
 {
 	double bits_gathered;
@@ -792,9 +846,6 @@ int main(int argc, char **argv)
 
 	openlog("rngd", 0, LOG_DAEMON);
 
-	/* Get the default watermark level for this platform */
-	arguments->fill_watermark = default_watermark();
-
 	/* Parsing of commandline parameters */
 	if (argp_parse(&argp, argc, argv, 0, 0, arguments) < 0)
 		return 1;
@@ -812,26 +863,25 @@ int main(int argc, char **argv)
 		pid_fd = write_pid_file(arguments->pid_file);
 		if (pid_fd < 0)
 			return 1;
-
 	}
 
 	if (arguments->list) {
 		int found = 0;
 		message(LOG_CONS|LOG_INFO, "Entropy sources that are available but disabled\n");
-		for (i=0; i < ENT_MAX; i++) 
+		for (i=0; i < ENT_MAX; i++)
 			if (entropy_sources[i].init && entropy_sources[i].disabled == true) {
 				found = 1;
 				message(LOG_CONS|LOG_INFO, "%d: %s (%s)\n", i,
 					entropy_sources[i].rng_name, entropy_sources[i].rng_sname);
+				entropy_sources[i].failed_init = false;
 			}
 		if (!found)
-			message(LOG_CONS|LOG_INFO, "None");
+			message(LOG_CONS|LOG_INFO, "None\n");
 		msg_squash = true;
 	} else
 		message(LOG_DAEMON|LOG_INFO, "Initializing available sources\n");
 
 	/* Init entropy sources */
-	
 	for (i=0; i < ENT_MAX; i++) {
 		ent_src = &entropy_sources[i];
 		if (ent_src->init && ent_src->disabled == false) {
@@ -840,9 +890,11 @@ int main(int argc, char **argv)
 				ent_src->fipsctx = malloc(sizeof(fips_ctx_t));
 				fips_init(ent_src->fipsctx, discard_initial_data(ent_src));
 				message_entsrc(ent_src, LOG_DAEMON|LOG_INFO, "Initialized\n");
+				ent_src->failed_init = false;
 			} else {
 				message_entsrc(ent_src, LOG_DAEMON|LOG_ERR, "Initialization Failed\n");
 				ent_src->disabled = true;
+				ent_src->failed_init = true;
 			}
 		}
 	}
@@ -851,20 +903,28 @@ int main(int argc, char **argv)
 		int rc = 1;
 		msg_squash = false;
 		message(LOG_CONS|LOG_INFO, "Available and enabled entropy sources:\n");
-		for (i=0; i < ENT_MAX; i++) 
+		for (i=0; i < ENT_MAX; i++)
 			if (entropy_sources[i].init && entropy_sources[i].disabled == false) {
 				rc = 1;
 				message(LOG_CONS|LOG_INFO, "%d: %s (%s)\n", i,
 					entropy_sources[i].rng_name, entropy_sources[i].rng_sname);
 			}
+		message(LOG_CONS|LOG_INFO, "Available entropy sources that failed initalization:\n");
+		for (i=0; i < ENT_MAX; i++)
+			if (entropy_sources[i].init && entropy_sources[i].disabled == true && entropy_sources[i].failed_init == true) {
+				rc = 1;
+				message(LOG_CONS|LOG_INFO, "%d: %s (%s)\n", i,
+					entropy_sources[i].rng_name, entropy_sources[i].rng_sname);
+			}
 
+		quiet = true;
 		close_all_entropy_sources();
 		return rc;
 	}
 
 	if (!ent_sources) {
 		message(LOG_DAEMON|LOG_ERR,
-			"can't open any entropy source");
+			"Can't open any entropy source\n");
 		message(LOG_DAEMON|LOG_ERR,
 			"Maybe RNG device modules are not loaded\n");
 		return 1;
@@ -882,6 +942,14 @@ int main(int argc, char **argv)
 		message(LOG_CONS|LOG_INFO, "Entering test mode...no entropy will "
 			"be delivered to the kernel\n");
 		signal(SIGALRM, alarm_signal);
+	} else {
+		/*
+		 * Use the alarm signal instead to wake us up on the force_reseed value
+		 */
+		if (arguments->force_reseed) {
+			signal(SIGALRM, force_reseed_signal);
+			alarm(arguments->force_reseed);
+		}
 	}
 
 	if (arguments->ignorefail)
